#include <boost/random/mersenne_twister.hpp>
#include <cmath>
#include "exponential.hpp"
#include "lognormal.hpp"

#define BOOST_TEST_MODULE "test::stats test suite"
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN
#include <boost/test/unit_test.hpp>

#ifndef NDEBUG
#   include <iostream>
#   define __TEST_DEBUG_EXPAND(x) x
#   define TEST_DEBUG_TRACE(x) std::cerr << "[Debug>> " << __TEST_DEBUG_EXPAND(x) << std::endl;
#else
#   define TEST_DEBUG_TRACE(x)
#endif // NDEBUG

static const double TOL(1.0e-5);
static const unsigned int SEED(123456);

BOOST_AUTO_TEST_CASE( exponential_distribution )
{
	TEST_DEBUG_TRACE( "Test case: exponential_distribution" );

	float lambda(10);
	test::stats::exponential_distribution<float> dist(lambda);
	boost::mt19937 rng(SEED);

	TEST_DEBUG_TRACE( "[exponential_distribution] lambda: " << dist.lambda() );
	BOOST_CHECK( std::fabs(dist.lambda() - lambda) <= TOL );

	TEST_DEBUG_TRACE( "[exponential_distribution] Mean: " << test::stats::mean(dist) );
	BOOST_CHECK( std::fabs(test::stats::mean(dist) - 1.0/lambda) <= TOL );

	TEST_DEBUG_TRACE( "[exponential_distribution] Variance: " << test::stats::variance(dist) );
	BOOST_CHECK( std::fabs(test::stats::variance(dist) - 1.0/(lambda*lambda)) <= TOL );

	TEST_DEBUG_TRACE( "[exponential_distribution] Standard Deviation: " << test::stats::standard_deviation(dist) );
	BOOST_CHECK( std::fabs(test::stats::standard_deviation(dist) - 1.0/lambda) <= TOL );

	TEST_DEBUG_TRACE( "[exponential_distribution] Rand#1: " << test::stats::rand(dist, rng) );
	BOOST_CHECK( test::stats::rand(dist, rng) > 0 );
	TEST_DEBUG_TRACE( "[exponential_distribution] Rand#2: " << test::stats::rand(dist, rng) );
	BOOST_CHECK( test::stats::rand(dist, rng) > 0 );
	TEST_DEBUG_TRACE( "[exponential_distribution] Rand#3: " << test::stats::rand(dist, rng) );
	BOOST_CHECK( test::stats::rand(dist, rng) > 0 );
}

BOOST_AUTO_TEST_CASE( lognormal_distribution )
{
	TEST_DEBUG_TRACE( "Test case: lognormal_distribution" );

	float location(0);
	float scale(1);
	test::stats::lognormal_distribution<float> dist(location, scale);
	boost::mt19937 rng(SEED);

	TEST_DEBUG_TRACE( "[lognormal_distribution] location: " << dist.location() );
	BOOST_CHECK( std::fabs(dist.location() - location) <= TOL );

	TEST_DEBUG_TRACE( "[lognormal_distribution] scale: " << dist.scale() );
	BOOST_CHECK( std::fabs(dist.scale() - scale) <= TOL );

	TEST_DEBUG_TRACE( "[lognormal_distribution] Mean: " << test::stats::mean(dist) );
	BOOST_CHECK( std::fabs(test::stats::mean(dist) - std::exp(location + std::pow(scale, 2)/2.0)) <= TOL );

	TEST_DEBUG_TRACE( "[lognormal_distribution] Variance: " << test::stats::variance(dist) );
	BOOST_CHECK( std::fabs(test::stats::variance(dist) - (std::exp(std::pow(scale, 2))-1)*std::exp(2.0*location + std::pow(scale, 2))) <= TOL );

	TEST_DEBUG_TRACE( "[lognormal_distribution] Standard Deviation: " << test::stats::standard_deviation(dist) );
	BOOST_CHECK( std::fabs(test::stats::standard_deviation(dist) - std::sqrt((std::exp(std::pow(scale, 2))-1)*std::exp(2.0*location + std::pow(scale, 2)))) <= TOL );

	TEST_DEBUG_TRACE( "[lognormal_distribution] Rand#1: " << test::stats::rand(dist, rng) );
	BOOST_CHECK( test::stats::rand(dist, rng) > 0 );
	TEST_DEBUG_TRACE( "[lognormal_distribution] Rand#2: " << test::stats::rand(dist, rng) );
	BOOST_CHECK( test::stats::rand(dist, rng) > 0 );
	TEST_DEBUG_TRACE( "[lognormal_distribution] Rand#3: " << test::stats::rand(dist, rng) );
	BOOST_CHECK( test::stats::rand(dist, rng) > 0 );
}
